# -*- encoding: utf-8 -*-

import csv
from django.core.urlresolvers import reverse
from django.contrib import admin
from django.utils.functional import update_wrapper
from django.http import Http404, HttpResponse, HttpResponseBadRequest

# Create a notation similar to queryset filter
SEPARATOR = u"__"

# String return if lookup property failed
EMPTY = u""

class ExportAdmin(admin.ModelAdmin):
    change_list_template = 'admin/export/change_list.html'

    def changelist_view(self, request, extra_context=None):
        """
        Add export URL for the template
        """
        if extra_context is None:
            extra_context = {}
        extra_context.update({
            'export_url' : reverse("admin:%s_%s_export" % (self.model._meta.app_label, self.model._meta.module_name))
        })
        return super(ExportAdmin, self).changelist_view(request, extra_context)


    def get_urls(self):
        from django.conf.urls.defaults import patterns, url

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)
            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.module_name

        urlpatterns = patterns('',
            url(r'^export/$',
                wrap(self.export),
                name='%s_%s_export' % info),
        )
        urlpatterns += super(ExportAdmin, self).get_urls()
        return urlpatterns

    def get_object_value(self, obj, header):
        """
        Function which work as a queryset filter access :
        Father -- Kid (One2One father related_name = 'child')
        father__name
        father__child__name
        It tries to resolve the property from header string, through
        linked object if needed. If it can't load any value, empty
        string is return.
        """
        segments = header.split(SEPARATOR)
        segments.reverse()
        prop = segments.pop()
        if len(segments) == 0:
            return getattr(obj, prop, EMPTY)
        else:
            try:
                child = getattr(obj, prop, None)
                return self.get_object_value(child, ".".join(segments))
            except:
                return EMPTY

    def get_default_csv_fields(self):
      fields_name = [f.name for f in  self.model._meta.fields]
      for fk in self.model._meta._related_objects_cache.keys():
          fields_name = fields_name + [u"%s%s%s" % (fk.var_name, SEPARATOR, f.name) for f in fk.model._meta.fields]
      return fields_name

    def export(self, request):
        """
        Generate HTTP response as CSV file from  csv_fields property.
        This property is a list of string, which should have queryset filter notation style (with X2 underscore).
        If there is no fields, the models is full introspected, stop at 1 depth level.
        """
        csv_fields = getattr(self, 'csv_fields', None)

        if csv_fields is None:
            csv_fields = self.get_default_csv_fields()

        qs = self.queryset(request)

        response = HttpResponse(mimetype='text/csv')
        response['Content-Disposition'] = 'attachment; filename=%s.csv' % unicode(self.model._meta.verbose_name_plural)

        writer = csv.writer(response)

        headers = []
        for attr in csv_fields:
            headers.append(attr)
        writer.writerow(headers)

        for o in qs:
            row = []
            for attr in csv_fields:
                value = self.get_object_value(o, attr)
                row.append(value)
            writer.writerow(row)

        return response

